--
-- control module (simulates SPIM control module)
--

library IEEE;
use  IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_arith.all;

entity control is


   port( signal Op : in std_logic_vector(0 to 5);
                signal RegDst : out std_logic;
                signal ALUSrc_D : out std_logic;
                signal MemtoReg_DD : out std_logic;
                signal RegWrite_D,RegWrite_DD,RegWrite_DDD : out std_logic;
                signal MemRead_DD : out std_logic;
                signal MemWrite_DD : out std_logic;
                signal Branch_D : out std_logic_vector(0 to 1);
				signal JumpR : out std_logic;
				Signal Jal : out std_logic;
				Signal Jump_D : out std_logic;

--				Signal Accept_Key : out std_logic;
				Signal Video_Write_DD : out std_logic;

                signal ALUop0_D : out std_logic;
                signal ALUop1_D : out std_logic;

                signal clock,reset,flush: in std_logic);
		
		
end control;


--
-- SPIM control architecture
--

architecture behavioral of control is
   

   signal Rformat, J, Jnl, Beqz, Bnez, Addi, Jr, Slti, Lw, Sw : std_logic;
   signal RegWrite : std_logic;
   signal MemtoReg : std_logic;
   signal MemRead : std_logic;
   signal MemWrite : std_logic;
   signal ALUSrc : std_logic;
   signal ALUOp0,ALUOp1: std_logic;
   signal Branch : std_logic_vector(0 to 1);
   signal Jump : std_logic;
   signal PutChar : std_logic;
   signal GetChar : std_logic;
   signal Video_Write : std_logic;


begin           -- behavior of SPIM control
------------------------------------------------------------------------------

		Rformat <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   ( NOT Op(3)) AND (NOT Op(4)) AND (NOT Op(5)));

		J       <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   ( NOT Op(3)) AND (    Op(4)) AND (NOT Op(5)));

		Jnl     <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   ( NOT Op(3)) AND (    Op(4)) AND (    Op(5)));

		Beqz    <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   (     Op(3)) AND (NOT Op(4)) AND (NOT Op(5)));

		Bnez    <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   (     Op(3)) AND (NOT Op(4)) AND (    Op(5)));

		Addi    <= ((NOT Op(0)) AND (NOT Op(1)) AND (NOT Op(2)) AND
				   (     Op(3)) AND (    Op(4)) AND (NOT Op(5)));

		Jr      <= ((NOT Op(0)) AND (    Op(1)) AND (NOT Op(2)) AND
				   (NOT  Op(3)) AND (NOT Op(4)) AND (NOT Op(5)));

		Slti    <= ((NOT Op(0)) AND (    Op(1)) AND (    Op(2)) AND
  				   (NOT  Op(3)) AND (    Op(4)) AND (NOT Op(5)));

		Lw      <= (    Op(0)   AND (NOT Op(1)) AND (NOT Op(2)) AND
				   (NOT Op(3))  AND (    Op(4)) AND (    Op(5)));

		Sw      <= (    Op(0)   AND (NOT Op(1)) AND (    Op(2)) AND
				   (NOT Op(3)) AND (    Op(4)) AND (    Op(5)));
-- Choosing 101100 for PutChar
		PutChar   <= (    Op(0)   AND (NOT Op(1)) AND (    Op(2)) AND
				   (    Op(3))  AND (NOT Op(4)) AND (NOT Op(5)));
-- Choosing 101101 For GetChar
		GetChar	<= (    Op(0)   AND (NOT Op(1)) AND (    Op(2)) AND
				   (    Op(3))  AND (NOT Op(4)) AND (    Op(5)));
				   
				   
-- needs to have the correct op code sequence				   
--	    Conv1  <=  (    Op(0)   AND (NOT Op(1)) AND (    Op(2)) AND
--				   (    Op(3))  AND (NOT Op(4)) AND (    Op(5)));
--	    Conv2  <=  (    Op(0)   AND (NOT Op(1)) AND (    Op(2)) AND
--				   (    Op(3))  AND (NOT Op(4)) AND (    Op(5)));
      
                
                RegDst <=  (Rformat or Addi or Slti) when flush='0' else '0';
                ALUSrc <=  Lw or Sw or Addi or Slti;
                MemtoReg <=  Lw when flush='0' else '0';
                RegWrite <=  (Rformat or Lw or Addi or Slti) when flush='0' else '0';
                MemRead <=  Lw when flush='0' else '0';
                MemWrite <=  Sw when flush='0' else '0';
                Branch(0) <= (Beqz or Bnez) when flush='0' else '0';
				Branch(1) <= (J or Bnez) when flush='0' else '0';
                ALUOp1 <=  Addi or Slti;
                ALUOp0 <=  Rformat or Addi;
				JumpR <= Jr when flush='0' else '0';
				Jal <= Jnl when flush='0' else '0';
				Jump <= (jr or jnl) when flush='0' else '0';
-- Need to check, changes here
--				Accept_Key <= GetChar when flush='0' else '0';
				Video_Write <= PutChar when flush='0' else '0';
  
Process

variable RegWrite_D3, RegWrite_D1, RegWrite_D2, MemtoReg_D, MemRead_D, MemWrite_D, Video_Write_D : std_logic;

begin
  wait until clock'event and clock='1';
   If reset='1' then
       RegWrite_D1 := '0';
       RegWrite_D2 := '0';
       RegWrite_D3 := '0';
	   Video_Write_D := '0';
       RegWrite_D <= '0';
       RegWrite_DD <= '0';
       RegWrite_DDD <= '0';      
       MemtoReg_DD <= '0';
       MemtoReg_D := '0';
	   MemRead_DD <= '0';
       MemRead_D := '0';
       MemWrite_DD <= '0';
       MemWrite_D := '0';
       ALUSrc_D <= '0';
       ALUOp0_D <= '0';
       ALUOp1_D <= '0';
	   Branch_D <= "00";
	   Jump_D <= '0';
--	   Accept_Key <= '0';
	   Video_Write_D := '0';
	   
   Else
	   regwrite_D3 := RegWrite_D2;
       RegWrite_D2 := RegWrite_D1;
       RegWrite_D1 := RegWrite;
       RegWrite_DDD <= RegWrite_D3;
	   RegWrite_DD <= RegWrite_D2;
	   RegWrite_D <= RegWrite_D1;
       MemtoReg_DD <= MemtoReg_D;
 	   MemtoReg_D := MemtoReg;
       MemRead_DD <= MemRead_D;
       MemRead_D := MemRead;
       MemWrite_DD <= MemWrite_D;
       MemWrite_D := MemWrite;
       ALUSrc_D <= ALUSrc;
       ALUOp0_D <= ALUOp0;
       ALUOp1_D <= ALUOp1;
	   Branch_D(0 to 1) <= Branch(0 to 1);
	   Jump_D <= Jump;
	   Video_Write_DD <= Video_Write_D;
	   Video_Write_D := Video_Write;
   end if;
end process;

end behavioral;

