--
--
-- Execute module (simulates SPIM (ALU) Execute module)
--
--

library altera,IEEE;
use altera.maxplus2.all;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_SIGNED.all;

entity  Execute is


        port(Readdata1 : in std_logic_vector(0 to 7);
		Readdata2 : in std_logic_vector(0 to 7);
	    rr2d_bus_DD : out std_logic_vector(0 to 7);
		Extend_D : in std_logic_vector(0 to 7);
		Func_D : in std_logic_vector(0 to 5);
		ALUOp0_D : in std_logic;
		ALUOp1_D : in std_logic; 
		ALUSrc_D : in std_logic;
		ALUResult_D : out std_logic_vector(0 to 7);
        video_data_D : out std_logic_vector(0 to 7);
	    wrd_bus : in std_logic_vector(0 to 7);
		ALUSelA_D, ALUSelB_D, char_mode : in std_logic_vector(0 to 1);
		clock,reset: in std_logic);

end Execute;

architecture behavioral of Execute is
 
Signal Ainput2,Binput2,Ainput,Binput, ALUresult_Dout : std_logic_vector(0 to 7);
Signal ResMux: std_logic_vector(0 to 7);
Signal ALUctl: std_logic_vector(0 to 2);
signal ALUResult : std_logic_vector(0 to 7);
signal ALTB : std_logic;
signal Ainput6,Binput6 : std_logic_vector(0 to 6);
signal video_reg : std_logic_vector(0 to 7);


begin
    Ainput(0 to 7) <= Ainput2(0 to 7);
    Binput(0 to 7) <= Binput2(0 to 7) WHEN (ALUSrc_D='0') ELSE Extend_D(0 to 7);

 	Ainput2(0 to 7) <= ALUResult_Dout(0 to 7) WHEN (ALUSelA_D(0 to 1) = "01")
					  ELSE wrd_bus(0 to 7) WHEN (ALUSelA_D(0 to 1) = "10")
					  ELSE readdata1(0 to 7); 

	Binput2(0 to 7) <= ALUResult_Dout(0 to 7) WHEN (ALUSelB_D(0 to 1) = "01")
					  ELSE wrd_bus(0 to 7) WHEN (ALUSelB_D(0 to 1) = "10")
					  ELSE readdata2(0 to 7); 
    
   video_reg(0 to 7) <= "0011" & Ainput2(0 to 3) when (char_mode(0 to 1) = "00") else
                        "0011" & Ainput2(4 to 7) when (char_mode(0 to 1) = "01") else
						Extend_D(0 to 7)         when (char_mode(0 to 1) = "10") else
						"00000000";

-- This is where the ALU control bits are set

	ALUctl(0) <= (ALUOp0_D AND (NOT ALUOp1_D) AND (NOT Func_D(0)) AND (NOT Func_D(1))
				 AND (NOT Func_D(2)) AND Func_D(3) AND (NOT FUNC_D(4)) 
				 AND ( NOT Func_D(5))) OR (ALUOp0_D AND (NOT ALUOp1_D)
				 AND Func_D(0) AND (NOT Func_D(1)) AND (NOT Func_D(3)) 
				 AND Func_D(4) AND (NOT Func_D(5))) OR ((NOT ALUOp0_D) AND ALUOp1_D);
	ALUctl(1) <= (Func_D(0) AND (NOT Func_D(1)) AND (NOT Func_D(2)) AND (NOT Func_D(3))
				 AND (NOT Func_D(5))) OR (Func_D(0) AND (NOT Func_D(1)) 
				 AND (NOT Func_D(3)) AND Func_D(4) AND (NOT Func_D(5))) 
				 OR ((NOT ALUOp0_D) OR ALUOp1_D);
	ALUctl(2) <= (ALUOp0_D AND (NOT ALUOp1_D) AND Func_D(0) AND (NOT Func_D(1))
				 AND Func_D(2) AND (NOT Func_D(3)) AND Func_D(4) AND (NOT Func_D(5)))
				 OR (ALUOp0_D AND (NOT ALUOp1_D) AND Func_D(0) AND (NOT Func_D(1))
				 AND (NOT Func_D(2)) AND Func_D(3) AND (NOT Func_D(4)) AND Func_D(5))
				 OR ((NOT ALUOp0_D) AND ALUOp1_D);
				 

-- Select ALU output        

--    ALUresult(0 to 7) <= To_stdlogicvector(B"0000000") & Resmux(7) WHEN ALUctl(0 to 2)="111"
--    ELSE ResMux(0 to 7);

Ainput6(0 to 6) <= Ainput(1 to 7);
Binput6(0 to 6) <= Binput(1 to 7);
ALTB <= '1' when Ainput6(0 to 6) < Binput6(0 to 6) else '0'; 

Process (ALUctl,Ainput,Binput)
  variable tempinput : std_logic_vector(0 to 7);

begin
-- with ALUctl(0 to 2) select
--	ResMux(0 to 7) <=
--		Ainput(0 to 7) AND Binput(0 to 7) WHEN "000",
--		Ainput(0 to 7) OR Binput(0 to 7) WHEN "001",
--		Ainput(0 to 7) + Binput(0 to 7) WHEN "010",
--		Ainput(0 to 7) - Binput(0 to 7) WHEN "110",
--		To_stdlogicvector(X"00") when others;

tempinput(6 to 7) := "00";
tempinput(0 to 5) := Ainput(2 to 7);

case ALUctl(0 to 2) is
 -- Select ALU operation
 -- ALU performs ALUresult = bus_A AND bus_B
WHEN "000" => ResMux(0 to 7) <= Ainput(0 to 7) AND Binput(0 to 7); 
 -- ALU performs ALUresult = bus_A OR bus_B
WHEN "001" => ResMux(0 to 7) <= Ainput(0 to 7) OR Binput(0 to 7);
 -- ALU performs ALUresult = bus_A + bus_B and ADDI and LW and SW
WHEN "010" => ResMux(0 to 7) <= Ainput(0 to 7) + Binput(0 to 7);
 -- ALU performs SLL
WHEN "100" => ResMux(0 to 7) <= tempinput(0 to 7);
--Ainput SLL Binput;
 -- ALU performs SLT and SLTI
--WHEN "111" => if (Ainput(0 to 7) < Binput(0 to 7)) THEN 
--			     ResMux(0 to 7) <= To_stdlogicvector(X"01");
--		  else
--			 Resmux(0 to 7) <= To_stdlogicvector(X"00");
--		  end if;
WHEN "111" => if (Ainput(0) ='1' and Binput(0)='0')
   							OR (Ainput(0) = Binput(0) AND ALTB='1') then
           				ResMux(0 to 7) <= "00000001";
		      else ResMux(0 to 7) <= "00000000";
			  end if;

WHEN Others => ResMux(0 to 7) <= "00000000";
end case;
end process;

ALUResult(0 to 7) <= ResMux(0 to 7);
--dff(Zero,reset,clock,Zero_D);
--dff(ALUResult,reset,clock,ALUResult_D);
--dff(AddResult,reset,clock,AddResult_D);

process
begin
  wait until clock'event and clock='1';
  if reset = '1' then
    ALUResult_D(0 to 7) <= "00000000";
    ALUResult_Dout(0 to 7) <= "00000000";
	rr2d_bus_DD(0 to 7) <= "00000000";
	video_data_D(0 to 7) <= "00000000";

  else
    rr2d_bus_DD(0 to 7) <= Binput2(0 to 7);
    ALUResult_D(0 to 7) <= ALUResult(0 to 7);
    ALUResult_Dout(0 to 7) <= ALUResult(0 to 7);
    video_data_D(0 to 7) <= video_reg(0 to 7);
  end if;
end process;
 
end behavioral;

