--
--  Idecode module (provides the register file for the DLX computer)
--

library altera,IEEE,lpm;
use altera.maxplus2.all;
use IEEE.STD_LOGIC_1164.all;
USE lpm.lpm_components.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_SIGNED.all;

entity Idecode is

        port(rr1d_bus_D : out std_logic_vector(0 to 7);
             rr2d_bus_D : out std_logic_vector(0 to 7);
             Instruction_D : in std_logic_vector(0 to 31);
             wrd_bus : in std_logic_vector(0 to 7);
             RegWrite_D,RegWrite_DD,RegWrite_DDD : in std_logic;
             RegDst : in std_logic;
             Zero_D : out std_logic;
             ADDResult_D,ADDResultBR_D : out std_logic_vector(0 to 7);
             PCadd_D : in std_logic_vector(0 to 7);
             Extend_D : out std_logic_vector(0 to 7);
             Func_D : out std_logic_vector(0 to 5);
             ALUSelA_D,ALUSelB_D : out std_logic_vector(0 to 1);
             ALUResult_D : in std_logic_vector(0 to 7);
			 switch : in std_logic_vector(0 to 7);
			 regvalue: out std_logic_vector(0 to 7);
	         jumpR,jal : in std_logic;
             char_mode_D : out std_logic_vector(0 to 1);
--			 Accept_Key : in std_logic;
--			 Key_Data  : in std_logic_vector (0 to 5);
--			 Key_Stroke : in std_logic;
             clock,reset: in std_logic);

end Idecode;

--
-- Idecode architecture
--

architecture behavioral of Idecode is

signal wraddress,wraddress_D,wraddress_DD,wraddress_DDD : std_logic_vector(0 to 4);
signal Extend : std_logic_vector(0 to 7);

signal reg1,reg2,reg3,reg4,reg5,reg6,reg7,reg8,reg9,reg10: std_logic_vector(0 to 7);
signal reg11,reg12,reg13,reg14,reg15 : std_logic_vector(0 to 7);
signal reg16,reg17,reg18,reg19: std_logic_vector(0 to 7);
signal reg20,reg21,reg22,reg23,reg24,reg25,reg26,reg27: std_logic_vector(0 to 7);
signal reg28,reg29,reg30,reg31 : std_logic_vector(0 to 7);

signal muxreg1,muxreg2,muxreg3,muxreg4,muxreg5,muxreg6,muxreg7,muxreg8 : std_logic_vector(0 to 7);
signal muxreg9,muxreg10,muxreg11,muxreg12,muxreg13,muxreg14,muxreg15: std_logic_vector(0 to 7);
signal muxreg16,muxreg17,muxreg18,muxreg19,muxreg20,muxreg21: std_logic_vector(0 to 7);
signal muxreg22,muxreg23,muxreg24,muxreg25,muxreg26,muxreg27,muxreg28,muxreg29: std_logic_vector(0 to 7);
signal muxreg30,muxreg31: std_logic_vector(0 to 7);

signal reg1wr,reg2wr,reg3wr,reg4wr,reg5wr,reg6wr,reg7wr,reg8wr,reg9wr : std_logic;
signal reg10wr,reg11wr,reg12wr,reg13wr,reg14wr,reg15wr,reg16wr : std_logic;
signal reg17wr,reg18wr,reg19wr,reg20wr,reg21wr,reg22wr,reg23wr,reg24wr : std_logic;
signal reg25wr,reg26wr,reg27wr,reg28wr,reg29wr,reg30wr: std_logic;

signal rr1add_bus,rr2add_bus,mem_addr: std_logic_vector(0 to 4);

signal Func: std_logic_vector(0 to 5);

signal rr1d_bus,rr2d_bus,rr1d_bus2,rr2d_bus2:std_logic_vector(0 to 7);
signal rr2d_bus3 : std_logic_vector(0 to 7);

signal Zero: std_logic;
signal AddResult, ResMux : std_logic_vector(0 to 7);
signal ALUSelA,ALUSelB, char_mode : std_logic_vector(0 to 1);
signal ADDResultBR : std_logic_vector(0 to 7);
signal RegWrite_DDDout : std_logic_vector(0 to 7);

signal Key_State : std_logic;
signal Wait_Key : std_logic;

begin
   rr1add_bus(0 to 4) <= Instruction_D(11 to 15);
   rr2add_bus(0 to 4) <= Instruction_D(16 to 20) when RegDst='1' else 
						        Instruction_D(6 to 10);
   wraddress(0 to 4)  <= Instruction_D(6 to 10);
   Func(0 to 5) <= Instruction_D(26 to 31);
   Extend(0 to 7) <= Instruction_D(24 to 31);
   char_mode(0 to 1) <= Instruction_D(9 to 10);
   


-- Need to add additional register support later to support 32 registers
-- Read Register Operations
   WITH rr1add_bus(0 to 4) SELECT
     rr1d_bus(0 to 7) <= "00000000"  WHEN "00000", 
		reg1(0 to 7)  WHEN "00001",
		reg2(0 to 7)  WHEN "00010",
  		reg3(0 to 7)  when "00011",
        reg4(0 to 7)  WHEN "00100",
        reg5(0 to 7)  WHEN "00101",
        reg6(0 to 7)  WHEN "00110",
 		reg7(0 to 7)  when "00111",
		reg8(0 to 7)  WHEN "01000",
		reg9(0 to 7)  when "01001",
	    reg10(0 to 7) when "01010",
		reg11(0 to 7) when "01011",
		reg12(0 to 7) when "01100",
		reg13(0 to 7) when "01101",
		reg14(0 to 7) when "01110",
		reg15(0 to 7) WHEN "01111",
		reg16(0 to 7) WHEN "10000",
		reg17(0 to 7) WHEN "10001",
		reg18(0 to 7) WHEN "10010",
		reg19(0 to 7) WHEN "10011",
		reg20(0 to 7) WHEN "10100",
		reg21(0 to 7) when "10101",
		reg22(0 to 7) when "10110",
		reg23(0 to 7) when "10111",
		reg24(0 to 7) WHEN "11000",
		reg25(0 to 7) WHEN "11001",
	    reg26(0 to 7) when "11010",
		reg27(0 to 7) when "11011",
		reg28(0 to 7) when "11100",
		reg29(0 to 7) WHEN "11101",
		reg30(0 to 7) WHEN "11110",
		reg31(0 to 7) WHEN "11111",
        "11111111" When others;
 
		 
      WITH rr2add_bus(0 to 4) SELECT
     rr2d_bus(0 to 7) <= "00000000"  WHEN "00000", 
		reg1(0 to 7)  WHEN "00001",
		reg2(0 to 7)  WHEN "00010",
  		reg3(0 to 7)  when "00011",
        reg4(0 to 7)  WHEN "00100",
        reg5(0 to 7)  WHEN "00101",
        reg6(0 to 7)  WHEN "00110",
 		reg7(0 to 7)  when "00111",
		reg8(0 to 7)  WHEN "01000",
		reg9(0 to 7)  when "01001",
	    reg10(0 to 7) when "01010",
		reg11(0 to 7) when "01011",
		reg12(0 to 7) when "01100",
		reg13(0 to 7) when "01101",
		reg14(0 to 7) when "01110",
		reg15(0 to 7) WHEN "01111",
		reg16(0 to 7) WHEN "10000",
		reg17(0 to 7) WHEN "10001",
		reg18(0 to 7) WHEN "10010",
		reg19(0 to 7) WHEN "10011",
		reg20(0 to 7) WHEN "10100",
		reg21(0 to 7) when "10101",
		reg22(0 to 7) when "10110",
		reg23(0 to 7) when "10111",
		reg24(0 to 7) WHEN "11000",
		reg25(0 to 7) WHEN "11001",
	    reg26(0 to 7) when "11010",
		reg27(0 to 7) when "11011",
		reg28(0 to 7) when "11100",
		reg29(0 to 7) WHEN "11101",
		reg30(0 to 7) WHEN "11110",
		reg31(0 to 7) WHEN "11111",
            "11111111" When others;
		    

muxreg1(0 to 7) <=reg1(0 to 7) WHEN reg1wr='0' ELSE wrd_bus;
reg1wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00001") AND (RegWrite_DDD='1')) ELSE '0';
muxreg2(0 to 7) <=reg2(0 to 7) WHEN reg2wr='0' ELSE wrd_bus;
reg2wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00010") AND (RegWrite_DDD='1')) ELSE '0';
muxreg3(0 to 7) <=reg3(0 to 7) WHEN reg3wr='0' ELSE wrd_bus;
reg3wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00011") AND (RegWrite_DDD='1')) ELSE '0';
muxreg4(0 to 7) <=reg4(0 to 7) WHEN reg4wr='0' ELSE wrd_bus;
reg4wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00100") AND (RegWrite_DDD='1')) ELSE '0';
muxreg5(0 to 7) <=reg5(0 to 7) WHEN reg5wr='0' ELSE wrd_bus;
reg5wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00101") AND (RegWrite_DDD='1')) ELSE '0';
muxreg6(0 to 7) <=reg6(0 to 7) WHEN reg6wr='0' ELSE wrd_bus;
reg6wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00110") AND (RegWrite_DDD='1')) ELSE '0';
muxreg7(0 to 7) <=reg7(0 to 7) WHEN reg7wr='0' ELSE wrd_bus;
reg7wr <= '1' WHEN ((wraddress_DDD(0 to 4)="00111") AND (RegWrite_DDD='1')) ELSE '0';
muxreg8(0 to 7) <=reg8(0 to 7) WHEN reg8wr='0' ELSE wrd_bus;
reg8wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01000") AND (RegWrite_DDD='1')) ELSE '0';
muxreg9(0 to 7) <=reg9(0 to 7) WHEN reg9wr='0' ELSE wrd_bus;
reg9wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01001") AND (RegWrite_DDD='1')) ELSE '0';
muxreg10(0 to 7) <=reg10(0 to 7) WHEN reg10wr='0' ELSE wrd_bus;
reg10wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01010") AND (RegWrite_DDD='1')) ELSE '0';
muxreg11(0 to 7) <=reg11(0 to 7) WHEN reg11wr='0' ELSE wrd_bus;
reg11wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01011") AND (RegWrite_DDD='1')) ELSE '0';
muxreg12(0 to 7) <=reg12(0 to 7) WHEN reg12wr='0' ELSE wrd_bus;
reg12wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01100") AND (RegWrite_DDD='1')) ELSE '0';
muxreg13(0 to 7) <=reg13(0 to 7) WHEN reg13wr='0' ELSE wrd_bus;
reg13wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01101") AND (RegWrite_DDD='1')) ELSE '0';
muxreg14(0 to 7) <=reg14(0 to 7) WHEN reg14wr='0' ELSE wrd_bus;
reg14wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01110") AND (RegWrite_DDD='1')) ELSE '0';
muxreg15(0 to 7) <=reg15(0 to 7) WHEN reg15wr='0' ELSE wrd_bus;
reg15wr <= '1' WHEN ((wraddress_DDD(0 to 4)="01111") AND (RegWrite_DDD='1')) ELSE '0';
muxreg16(0 to 7) <=reg16(0 to 7) WHEN reg16wr='0' ELSE wrd_bus;
reg16wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10000") AND (RegWrite_DDD='1')) ELSE '0';
muxreg17(0 to 7) <=reg17(0 to 7) WHEN reg17wr='0' ELSE wrd_bus;
reg17wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10001") AND (RegWrite_DDD='1')) ELSE '0';
muxreg18(0 to 7) <=reg18(0 to 7) WHEN reg18wr='0' ELSE wrd_bus;
reg18wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10010") AND (RegWrite_DDD='1')) ELSE '0';
muxreg19(0 to 7) <=reg19(0 to 7) WHEN reg19wr='0' ELSE wrd_bus;
reg19wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10011") AND (RegWrite_DDD='1')) ELSE '0';
muxreg20(0 to 7) <=reg20(0 to 7) WHEN reg20wr='0' ELSE wrd_bus;
reg20wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10100") AND (RegWrite_DDD='1')) ELSE '0';
muxreg21(0 to 7) <=reg21(0 to 7) WHEN reg21wr='0' ELSE wrd_bus;
reg21wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10101") AND (RegWrite_DDD='1')) ELSE '0';
muxreg22(0 to 7) <=reg22(0 to 7) WHEN reg22wr='0' ELSE wrd_bus;
reg22wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10110") AND (RegWrite_DDD='1')) ELSE '0';
muxreg23(0 to 7) <=reg23(0 to 7) WHEN reg23wr='0' ELSE wrd_bus;
reg23wr <= '1' WHEN ((wraddress_DDD(0 to 4)="10111") AND (RegWrite_DDD='1')) ELSE '0';
muxreg24(0 to 7) <=reg24(0 to 7) WHEN reg24wr='0' ELSE wrd_bus;
reg24wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11000") AND (RegWrite_DDD='1')) ELSE '0';
muxreg25(0 to 7) <=reg25(0 to 7) WHEN reg25wr='0' ELSE wrd_bus;
reg25wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11001") AND (RegWrite_DDD='1')) ELSE '0';
muxreg26(0 to 7) <=reg26(0 to 7) WHEN reg26wr='0' ELSE wrd_bus;
reg26wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11010") AND (RegWrite_DDD='1')) ELSE '0';
muxreg27(0 to 7) <=reg27(0 to 7) WHEN reg27wr='0' ELSE wrd_bus;
reg27wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11011") AND (RegWrite_DDD='1')) ELSE '0';
muxreg28(0 to 7) <=reg28(0 to 7) WHEN reg28wr='0' ELSE wrd_bus;
reg28wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11100") AND (RegWrite_DDD='1')) ELSE '0';
muxreg29(0 to 7) <=reg29(0 to 7) WHEN reg29wr='0' ELSE wrd_bus;
reg29wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11101") AND (RegWrite_DDD='1')) ELSE '0';
muxreg30(0 to 7) <=reg30(0 to 7) WHEN reg30wr='0' ELSE wrd_bus;
reg30wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11110") AND (RegWrite_DDD='1')) ELSE '0';

--muxreg31(0 to 7) <=reg31(0 to 7) WHEN reg31wr='0' ELSE wrd_bus;
--reg31wr <= '1' WHEN ((wraddress_DDD(0 to 4)="11111") AND (RegWrite_DDD='1')) ELSE '0';


--  ( For 8-bit data path model)  


rr1d_bus2(0 to 7) <= wrd_bus(0 to 7) when
     (wraddress_DDD(0 to 4) = rr1add_bus(0 to 4) AND RegWrite_DDD = '1') 
	  else rr1d_bus(0 to 7);
--SW forwarding still applies since this is the Register to store
rr2d_bus2(0 to 7) <= wrd_bus(0 to 7) WHEN
     ((wraddress_DDD(0 to 4) = rr2add_bus(0 to 4)) AND (RegWrite_DDD = '1'))
	  else rr2d_bus(0 to 7);

rr2d_bus3(0 to 7) <= ALUResult_D(0 to 7) WHEN ((wraddress_DD(0 to 4) = rr2add_bus(0 to 4))
		   AND (RegWrite_D = '1'))
	  else rr2d_bus2(0 to 7);

RegWrite_DDDout(0 to 7) <= "00000" & RegWrite_D & RegWrite_DD & RegWrite_DDD;
 
 WITH switch(2 to 7) SELECT
     regvalue(0 to 7) <= reg1(0 to 7)  WHEN "000001",
		reg2(0 to 7)  WHEN "000010",
  		reg3(0 to 7)  when "000011",
        reg4(0 to 7)  WHEN "000100",
        reg5(0 to 7)  WHEN "000101",
        reg6(0 to 7)  WHEN "000110",
 		reg7(0 to 7)  when "000111",
		reg8(0 to 7)  WHEN "001000",
		reg9(0 to 7)  when "001001",
	    reg10(0 to 7) when "001010",
		reg11(0 to 7) when "001011",
		reg12(0 to 7) when "001100",
		reg13(0 to 7) when "001101",
		reg14(0 to 7) when "001110",
		reg15(0 to 7) WHEN "001111",
		reg16(0 to 7) WHEN "010000",
		reg17(0 to 7) WHEN "010001",
		reg18(0 to 7) WHEN "010010",
		reg19(0 to 7) WHEN "010011",
		reg20(0 to 7) WHEN "010100",
		reg21(0 to 7) when "010101",
		reg22(0 to 7) when "010110",
		reg23(0 to 7) when "010111",
		reg24(0 to 7) WHEN "011000",
		reg25(0 to 7) WHEN "011001",
	    reg26(0 to 7) when "011010",
		reg27(0 to 7) when "011011",
		reg28(0 to 7) when "011100",
		reg29(0 to 7) WHEN "011101",
		reg30(0 to 7) WHEN "011110",
		reg31(0 to 7) WHEN "011111",

        regWrite_DDDout(0 to 7) WHEN "100000",
        wraddress_D(0 to 4)     When "100001",
		wraddress_DD(0 to 4)    when "100010",
		wraddress_DDD(0 to 4)   when "100011",
        AddResultBR(0 to 7)     WHEN "100100",
        AddResult(0 to 7)		WHEN "100101",
		"00000000" WHEN OTHERS;


-- 11/8 mikes 1:49 PM I think this is a problem but I'm not going to fix it right
-- now.  This following code doesn't use the forwarding provided by the above muxes.

--ResMux(0 to 7) <= rr1d_bus2(0 to 7) - rr2d_bus2(0 to 7);
--Zero <= '1' WHEN ResMux(0 to 7) = "00000000" ELSE '0';

Zero <='1' When rr2d_bus3="00000000" else '0';
muxreg31(0 to 7) <= PCadd_D(0 to 7) when Jal='1' else reg31(0 to 7);
AddresultBR(0 to 7) <= Instruction_D(24 to 31) when jal = '1'
	ELSE Reg31(0 to 7) when jumpr = '1'
	ELSE "00000000";
AddResult(0 to 7) <= Extend(0 to 7);      

--ALUSelA should not use forwarding for LW/SW since the second data line is actually
--the memory address in disguise!
ALUSelA(0 to 1) <= "01" when 
			(RegWrite_D = '1' and (wraddress_D(0 to 4) = rr1add_bus(0 to 4)) AND 
			 rr1add_bus(0 to 4) /= "00000")
		else "10" when 
	        (RegWrite_DD = '1' and (wraddress_DD(0 to 4) = rr1add_bus(0 to 4)) AND
			(RegWrite_D = '0' or (RegWrite_D = '1' AND 
			 wraddress_D(0 to 4) /= rr1add_bus(0 to 4)))
			 AND rr1add_bus(0 to 4)	/= "00000")
        else "00";

ALUSelB(0 to 1) <= "01" when 
			(RegWrite_D = '1' and (wraddress_D(0 to 4) = rr2add_bus(0 to 4)) AND 
			 rr2add_bus(0 to 4) /= "00000")
		else "10" when 
	        (RegWrite_DD = '1' and (wraddress_DD(0 to 4) = rr2add_bus(0 to 4)) AND
			(RegWrite_D = '0' or (RegWrite_D = '1' AND 
			 wraddress_D(0 to 4) /= rr2add_bus(0 to 4)))
			 AND rr2add_bus(0 to 4)	/= "00000")
        else "00";




Process
Begin
  wait until clock'event and clock='1';
  if reset='1' then
    Addresult_D(0 to 7) <= "00000000";
    AddResultBR_D(0 to 7) <= "00000000";
    ALUSelA_D <= "00";
    ALUSelB_D <= "00";
  else
   --Write address is written back here and not needed in other modules
  wraddress_DDD(0 to 4) <= wraddress_DD(0 to 4); -- to get instruction back to idecode for write back
  wraddress_DD(0 to 4) <= wraddress_D(0 to 4);   -- for when instruction is in memory
  wraddress_D(0 to 4) <= wraddress(0 to 4);      -- for when instruction is in execute
  rr1d_bus_D(0 to 7) <= rr1d_bus2(0 to 7);
  rr2d_bus_D(0 to 7) <= rr2d_bus2(0 to 7);

  Extend_D(0 to 7) <= Extend(0 to 7);
  Func_D(0 to 5) <= Func(0 to 5);
  AddResult_D(0 to 7) <= AddResult(0 to 7);
  AddResultBR_D(0 to 7) <= AddResultBR(0 to 7);

  ALUSelA_D(0 to 1) <= ALUSelA(0 to 1);
  ALUSelB_D(0 to 1) <= ALUSelB(0 to 1);
  end if;
end process;

Process
Begin
 wait until clock'event and clock='1';
 if reset='1' then
  Zero_D <= '0';
 
 --These initial values were added for testing of the Altera Board and chip only
  --They should be removed later.
  reg1(0 to 7)  <= "00000001";
  reg2(0 to 7)  <= "00000010";
  reg3(0 to 7)  <= "00000011";
  reg5(0 to 7)  <= "00000101";
  reg8(0 to 7)  <= "00001000";
  reg17(0 to 7) <= "00010001";
  reg19(0 to 7) <= "00010011";
  reg20(0 to 7) <= "00010100";
  reg24(0 to 7) <= "00011000";
  reg25(0 to 7) <= "00011001";

 else 
  char_mode_D(0 to 1) <= char_mode(0 to 1);
  Zero_D <= Zero;
  reg1(0 to 7)<=muxreg1(0 to 7);
  reg2(0 to 7)<=muxreg2(0 to 7);
  reg3(0 to 7)<=muxreg3(0 to 7);  
  reg4(0 to 7)<=muxreg4(0 to 7);
  reg5(0 to 7)<=muxreg5(0 to 7);
  reg6(0 to 7)<=muxreg6(0 to 7);
  reg7(0 to 7)<=muxreg7(0 to 7);
  reg8(0 to 7)<=muxreg8(0 to 7);
  reg9(0 to 7)<=muxreg9(0 to 7);
  reg10(0 to 7)<=muxreg10(0 to 7);
  reg11(0 to 7)<=muxreg11(0 to 7);
  reg12(0 to 7)<=muxreg12(0 to 7);
  reg13(0 to 7)<=muxreg13(0 to 7);
  reg14(0 to 7)<=muxreg14(0 to 7);
  reg15(0 to 7)<=muxreg15(0 to 7);
  reg16(0 to 7)<=muxreg16(0 to 7);
  reg17(0 to 7)<=muxreg17(0 to 7);
  reg18(0 to 7)<=muxreg18(0 to 7);
  reg19(0 to 7)<=muxreg19(0 to 7);
  reg20(0 to 7)<=muxreg20(0 to 7);
  reg21(0 to 7)<=muxreg21(0 to 7);
  reg22(0 to 7)<=muxreg22(0 to 7);
  reg23(0 to 7)<=muxreg23(0 to 7);
  reg24(0 to 7)<=muxreg24(0 to 7);
  reg25(0 to 7)<=muxreg25(0 to 7);
  reg26(0 to 7)<=muxreg26(0 to 7);
  reg27(0 to 7)<=muxreg27(0 to 7);
  reg28(0 to 7)<=muxreg28(0 to 7);
  reg29(0 to 7)<=muxreg29(0 to 7);
  reg30(0 to 7)<=muxreg30(0 to 7);
  reg31(0 to 7)<=muxreg31(0 to 7);  
 end if;
end process;


-- Little process that will initialize the different flags once a GetChar operation
-- has been called. Key_State will be the position of the character that has been
-- entered

--process(Accept_Key)
--  begin
--	Key_State <= '0';
--	Wait_Key <= '1';
--end process;

-- Supposes that two numbers are entered every time. Thus the Key_State flag keeps
-- count of position of where the character is being entered ( set to zero means 
-- most 'significant bit', set to 1 means least significant) 

-- Problem ... Reg30 is 0 to 7, Key_Data is 5 downto 0, need two sets of key
-- therefore 10 bits... So what value is going to be passed to Key_Data?
-- Should be the Hex value of the numbers...

--process(Key_Stroke)
--  begin
--    if Wait_Key ='1' then 
--		if Key_State ='0' then
--			Reg30(2 to 3) <= Key_data(1 downto 0);
--			Key_State <='1';
--		else
--			Reg30(0 to 1) <= Key_Data(1 downto 0);
--			
--		end if;
--	end if;
--end process;


end behavioral;

