--
--
-- Ifetch module (provides the PC and instruction memory for the SPIM computer)

library altera,IEEE,lpm;
use altera.maxplus2.all;
use IEEE.STD_LOGIC_1164.all;
USE lpm.lpm_components.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_SIGNED.all;


-- We can have up to 256 instructions with a PC address of 10 bits
-- however, we chose to only use 128 instructions at this time

entity Ifetch is

  Generic(ADDR_WIDTH : integer := 8; DATA_WIDTH: integer := 32);

        port(signal Instruction_D : out std_logic_vector(0 to 31);
        signal PCadd_D : out  std_logic_vector(0 to 7);
        signal Addresult_D,AddResultBR_D: in std_logic_vector(0 to 7);
        signal Branch_D : in std_logic_vector(0 to 1);
        signal clock, reset : in std_logic;
        signal Zero_D,jump_D : in std_logic;
        signal flush : out std_logic;
        signal PCout : out std_logic_vector(0 to 7));

end Ifetch;

--
-- Ifetch architecture
--

architecture behavioral of Ifetch is

        signal Instruction : std_logic_vector(0 to 31);
        signal PC,NextPC,PCCurr : std_logic_vector(0 to 7);
        signal PCtemp,addtemp : std_logic_vector(0 to 7);
        signal NextPCadd : std_logic_vector(0 to 8);
        signal tempflush : std_logic;

  -- Signals needed for RAM
--signal nowrite: std_logic;

	begin

 				
 inst_ram: lpm_rom
      GENERIC MAP (lpm_widthad => ADDR_WIDTH,
         lpm_outdata => "UNREGISTERED",
         lpm_address_control => "UNREGISTERED",
         lpm_file => "instruct.mif",
         lpm_width => DATA_WIDTH)
      PORT MAP (address => PCCurr(0 to 7), 
                --inclock => clock,
                q => Instruction(0 to 31));
 				

-- Increment PC by 4        
	PCout(0 to 7) <= PCCurr(0 to 7);

      NextPCAdd(0 to 8) <= PCCurr(0 to 7) + 1;
      NextPC(0 to 7) <= NextPCAdd(1 to 8);

    tempflush <= '1' when (Branch_D(0 to 1) = "01") 
             or (Branch_D(0 to 1) = "10" And Zero_D='1')
             or (Branch_D(0 to 1) = "11" and Zero_D='0')
             or jump_D='1'
            else '0';

PCCurr(0 to 7) <= AddResultBR_D(0 to 7) when jump_D='1' 
   				else AddResult_D(0 to 7) when tempflush='1'
   				else PC(0 to 7);
  
flush <= tempflush;

-- Load next PC
process
begin
  wait until clock'event and clock='1';
  if reset='1' then
     PC <= "00000000";
     PCAdd_D(0 to 7) <= "00000000";
     Instruction_D(0 to 31) <= "00000000000000000000000000000000";
  else
    PC(0 to 7) <= NextPC(0 to 7);
    PCAdd_D(0 to 7) <= NextPC(0 to 7);
    Instruction_D(0 to 31) <= Instruction(0 to 31);
  end if;
 end process;

end behavioral;
