-- TOP_SPIM module
--
-- VHDL synthesis and simulation model of MIPS machine
-- as described in chapter 5 of Patterson and Hennessey
-- NOTE: Data paths limited to 8 bits to speed synthesis and
-- simulation.  Registers limited to 8 bits and $R0..$R7
-- Program and Data memory limited to locations 0..7

Library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity TOP_SPIM is

port(reset,clock: in std_logic;
      PC : out std_logic_vector(0 to 7);
	  Out_Inst : out std_logic_vector(0 to 31);
      Video_Data_D : out std_logic_vector(0 to 7);
	  Video_Write_DD : out std_logic;
      switch : in std_logic_vector(0 to 7));
end TOP_SPIM;


architecture BEHAVIORAL of TOP_SPIM is

   component Ifetch

   port(Instruction_D: out std_logic_vector(0 to 31);
        PCadd_D : out  std_logic_vector(0 to 7);
        Addresult_D,AddResultBR_D : in std_logic_vector(0 to 7);
        Branch_D : in std_logic_vector(0 to 1);
        clock,reset : in std_logic;
        Zero_D,jump_D : in std_logic;
        flush : out std_logic;
        PCout : out std_logic_vector(0 to 7));

   end component;

   component Idecode

   port(rr1d_bus_D : out std_logic_vector(0 to 7);
        rr2d_bus_D: out std_logic_vector(0 to 7);
        Instruction_D : in std_logic_vector(0 to 31);
        wrd_bus : in std_logic_vector(0 to 7);
        RegWrite_D,RegWrite_DD,RegWrite_DDD  : in std_logic;
        RegDst : in std_logic;
        Zero_D : out std_logic;
        ADDResult_D,AddResultBR_D : out std_logic_vector(0 to 7);
        PCadd_D : in std_logic_vector(0 to 7);
        Extend_D : out std_logic_vector(0 to 7);
        Func_D : out std_logic_vector(0 to 5);
        ALUSelA_D,ALUSelB_D : out std_logic_vector(0 to 1);
        ALUResult_D : in std_logic_vector(0 to 7);
        regvalue : out std_logic_vector(0 to 7);
        switch: in std_logic_vector(0 to 7);
        jumpr,jal : in std_logic;
        char_mode_D : out std_logic_vector(0 to 1);
--      Accept_Key : in std_logic;
--		Key_Data  : in std_logic_vector (0 to 5);
--		Key_Stroke : in std_logic;
        clock,reset: in std_logic);

	end component;

   component control

   port( Op : in std_logic_vector(0 to 5);
                RegDst : out std_logic;
                ALUSrc_D : out std_logic;
                MemtoReg_DD : out std_logic;
                RegWrite_D,RegWrite_DD,RegWrite_DDD : out std_logic;
                MemRead_DD : out std_logic;
                MemWrite_DD : out std_logic;
                Branch_D: out std_logic_vector(0 to 1);
                ALUop0_D : out std_logic;
                ALUop1_D : out std_logic;
                JumpR,jal,jump_D : out std_logic;
--				Accept_Key : out std_logic;
				Video_Write_DD : out std_logic;
                clock,reset,flush: in std_logic);
	
	 end component;

   component  Execute

   port(Readdata1 : in std_logic_vector(0 to 7);
        Readdata2 : in std_logic_vector(0 to 7);
        rr2d_bus_DD : out std_logic_vector(0 to 7);
        Extend_D : in std_logic_vector(0 to 7);
        Func_D : in std_logic_vector(0 to 5);
        ALUOp0_D : in std_logic;
        ALUOp1_D : in std_logic; 
        ALUSrc_D : in std_logic;
        ALUResult_D : out std_logic_vector(0 to 7);
        wrd_bus : in std_logic_vector(0 to 7);
        ALUSelA_D,ALUSelB_D, char_mode : in std_logic_vector(0 to 1);
        video_data_D : out std_logic_vector(0 to 7);
        clock,reset: in std_logic);

	end component;

   component dmemory

   port(rd_bus_D : out std_logic_vector(0 to 7);
        ra_bus : in std_logic_vector(0 to 7);
        wd_bus : in std_logic_vector(0 to 7);
        MemRead_DD, Memwrite_DD, MemtoReg_DD : in std_logic;
        clock,reset: in std_logic);

   end component;

   signal PCadd_D :  std_logic_vector(0 to 7);
   signal rr1d_bus_D,rr2d_bus_D : std_logic_vector(0 to 7);
   signal rr2d_bus_DD : std_logic_vector(0 to 7);
   signal Extend_D : std_logic_vector(0 to 7);
   signal Func_D : std_logic_vector(0 to 5);
   signal Addresult_D,AddResultBR_D : std_logic_vector(0 to 7);
   signal ALUresult_D : std_logic_vector(0 to 7);
   signal Branch_D: std_logic_vector(0 to 1);
   signal Zero_D,jump_d : std_logic;
   signal wrd_bus : std_logic_vector(0 to 7);
   signal RegWrite_D,RegWrite_DD,RegWrite_DDD : std_logic;
   signal RegDst :  std_logic;
   signal ALUSrc_D :  std_logic;
   signal MemtoReg_DD :  std_logic;
   signal MemRead_DD :  std_logic;
   signal MemWrite_DD :  std_logic;
   signal ALUop_D :  std_logic_vector(0 to 1);
   signal Instruction_D: std_logic_vector(0 to 31);
   signal ALUSelA_D : std_logic_vector(0 to 1);
   signal ALUSelB_D : std_logic_vector(0 to 1);
   signal jumpr,jal,flush : std_logic;
   signal char_mode : std_logic_vector(0 to 1);
   signal regvalue : std_logic_vector(0 to 7);
--   signal accept_key : std_logic;




   begin
   Out_Inst(0 to 31) <= Instruction_D(0 to 31);

   IFE : Ifetch

   port map (Instruction_D => Instruction_D(0 to 31),
             PCadd_D => PCadd_D(0 to 7),
             Addresult_D(0 to 7) => Addresult_D(0 to 7),
             AddResultBR_D(0 to 7) => AddResultBR_D(0 to 7),
             Branch_D(0 to 1) => Branch_D(0 to 1),
             clock => clock, 
             reset => reset,
             Zero_D => Zero_D,
             Jump_D => Jump_D,
             flush => flush,
             PCout => PC(0 to 7));

   ID : Idecode

   port map (rr1d_bus_D => rr1d_bus_D(0 to 7),
             rr2d_bus_D => rr2d_bus_D(0 to 7),
             Instruction_D(0 to 31) => Instruction_D(0 to 31),
             wrd_bus(0 to 7) => wrd_bus(0 to 7),
             RegWrite_D => RegWrite_D,
             RegWrite_DD => RegWrite_DD,
             RegWrite_DDD => RegWrite_DDD,
             RegDst => RegDst,
             Zero_D => Zero_D,
             ADDResult_D => ADDResult_D(0 to 7),
             AddResultBR_D => ADDResultBR_D(0 to 7),
             PCadd_D(0 to 7) => PCadd_D(0 to 7),
             Extend_D => Extend_D(0 to 7),
             Func_D => Func_D(0 to 5),
             ALUSelA_D => ALUSelA_D(0 to 1),
             ALUSelB_D => ALUSelB_D(0 to 1),
             ALUResult_D(0 to 7) => ALUResult_D(0 to 7),
             jumpr => jumpr,
             jal => jal,
             regvalue => regvalue(0 to 7),
             switch => switch(0 to 7),
			 char_mode_D => char_mode(0 to 1),
             clock => clock, reset => reset);

   CTL:   control

   port map (Op(0 to 5) => Instruction_D(0 to 5),
		RegDst => RegDst,
		ALUSrc_D => ALUSrc_D,
		MemtoReg_DD => MemtoReg_DD,
		RegWrite_D => RegWrite_D,
		RegWrite_DD => RegWrite_DD,
		RegWrite_DDD => RegWrite_DDD,
		MemRead_DD => MemRead_DD,
		MemWrite_DD => MemWrite_DD,
		Branch_D => Branch_D(0 to 1),
		ALUop0_D => ALUop_D(0),
		ALUop1_D => ALUop_D(1),
        jump_D => jump_D,
        jumpr => jumpr,
        jal => jal,
--		Accept_key => accept_key,
		Video_Write_DD => video_write_DD,
		clock => clock, 
        reset => reset,
        flush => flush);

   EXE:  Execute


   port map (Readdata1(0 to 7) => rr1d_bus_D(0 to 7),
		Readdata2(0 to 7) => rr2d_bus_D(0 to 7),
        rr2d_bus_DD => rr2d_bus_DD(0 to 7),
		Extend_D(0 to 7) => Extend_D(0 to 7),
        Func_D(0 to 5) => Func_D(0 to 5),
		ALUOp0_D => ALUop_D(0),
		ALUOp1_D => ALUop_D(1), 
		ALUSrc_D => ALUSrc_D,
		ALUResult_D => ALUResult_D(0 to 7),
        wrd_bus(0 to 7) => wrd_bus(0 to 7),
        ALUSelA_D(0 to 1) => ALUSelA_D(0 to 1),
        ALUSelB_D(0 to 1) => ALUSelB_D(0 to 1),
		Video_data_D => video_data_D(0 to 7),
		char_mode => char_mode(0 to 1),
		clock => clock, reset => reset);

   MEM:  dmemory

   port map (rd_bus_D => wrd_bus(0 to 7),
			ra_bus(0 to 7) => ALUResult_D(0 to 7),
			wd_bus(0 to 7) => rr2d_bus_DD(0 to 7),
			MemRead_DD => MemRead_DD, 
			Memwrite_DD => MemWrite_DD, 
			MemtoReg_DD => MemtoReg_DD,
       	 	clock => clock, 
        	reset => reset);

end behavioral;

--configuration SPIM of TOP_SPIM is
--for BEHAVIORAL
-- for IFE: Ifetch
--    use entity work.IFETCH(BEHAVIORAL);
-- end for;
-- for ID: Idecode
--    use entity work.IDECODE(BEHAVIORAL);
-- end for;
-- for CTL: control
--    use entity work.CONTROL(BEHAVIORAL);
-- end for;
-- for EXE: execute
--    use entity work.EXECUTE(BEHAVIORAL);
-- end for;
-- for MEM: dmemory
--    use entity work.DMEMORY(BEHAVIORAL);
-- end for;
--end for;
--end SPIM;


